@@ -427,33 +427,44 @@ void calcReplayerVars(int32_t audioFreq)
 	}
 }
 
-// for piano in Instr. Ed. (values outside 0..95 can happen)
-int32_t getPianoKey(uint16_t period, int8_t finetune, int8_t relativeNote)
+// for piano in Instr. Ed. (values outside 0..95 can and will happen)
+int32_t getPianoKey(int32_t period, int8_t finetune, int8_t relativeNote)
 {
-	assert(note2PeriodLUT != NULL);
-	if (period > note2PeriodLUT[0])
-		return -1; // outside left piano edge
-
-	finetune = ((int8_t)finetune >> 3) + 16; // -128..127 -> 0..31
-
-	int32_t hiPeriod = 10*12*16;
-	int32_t loPeriod = 0;
-
-	for (int32_t i = 0; i < 7; i++)
+	if (audio.linearPeriodsFlag)
 	{
-		const int32_t tmpPeriod = (((loPeriod + hiPeriod) >> 1) & ~15) + finetune;
+		period >>= 2;
+		period += (int8_t)finetune >> 3;
 
-		int32_t lookUp = tmpPeriod - 16;
-		if (lookUp < 0)
-			lookUp = 0;
-
-		if (period >= note2PeriodLUT[lookUp])
-			hiPeriod = (tmpPeriod - finetune) & ~15;
-		else
-			loPeriod = (tmpPeriod - finetune) & ~15;
+		return (((10*12*16) - period) >> 4) - relativeNote;
 	}
+	else
+	{
+		// amiga periods, requires slower method (binary search in amiga period LUT)
 
-	return (loPeriod >> 4) - relativeNote;
+		if (period > amigaPeriodLUT[0])
+			return -1; // outside left piano edge
+
+		finetune = ((int8_t)finetune >> 3) + 16; // -128..127 -> 0..31
+
+		int32_t hiPeriod = 10*12*16;
+		int32_t loPeriod = 0;
+
+		for (int32_t i = 0; i < 7; i++)
+		{
+			const int32_t tmpPeriod = (((loPeriod + hiPeriod) >> 1) & ~15) + finetune;
+
+			int32_t lookUp = tmpPeriod - 16;
+			if (lookUp < 0)
+				lookUp = 0;
+
+			if (period >= amigaPeriodLUT[lookUp])
+				hiPeriod = (tmpPeriod - finetune) & ~15;
+			else
+				loPeriod = (tmpPeriod - finetune) & ~15;
+		}
+
+		return (loPeriod >> 4) - relativeNote;
+	}
 }
 
 void triggerNote(uint8_t note, uint8_t efx, uint8_t efxData, channel_t *ch)
@@ -652,22 +663,22 @@ static void patternDelay(channel_t *ch, uint8_t param)
 
 static const efxRoutine EJumpTab_TickZero[16] =
 {
-	dummy,			// 0
+	dummy,				// 0
 	finePitchSlideUp,	// 1
 	finePitchSlideDown,	// 2
 	setPortamentoCtrl,	// 3
 	setVibratoCtrl,		// 4
-	dummy,			// 5
+	dummy,				// 5
 	patternLoop,		// 6
 	setTremoloCtrl,		// 7
-	dummy,			// 8
-	dummy,			// 9
+	dummy,				// 8
+	dummy,				// 9
 	fineVolSlideUp,		// A
 	fineVolFineDown,	// B
-	noteCut0,		// C
-	dummy,			// D
+	noteCut0,			// C
+	dummy,				// D
 	patternDelay,		// E
-	dummy			// F
+	dummy				// F
 };
 
 static void E_Effects_TickZero(channel_t *ch, uint8_t param)
@@ -677,8 +688,10 @@ static void E_Effects_TickZero(channel_t *ch, uint8_t param)
 
 	if (ch->channelOff) // channel is muted, only handle certain E effects
 	{
-		     if (efx == 0x6) patternLoop(ch, param);
-		else if (efx == 0xE) patternDelay(ch, param);
+		if (efx == 0x6)
+			patternLoop(ch, param);
+		else if (efx == 0xE)
+			patternDelay(ch, param);
 
 		return;
 	}
@@ -896,42 +909,42 @@ static void setEnvelopePos(channel_t *ch, uint8_t param)
 
 static const efxRoutine JumpTab_TickZero[36] =
 {
-	dummy,			// 0
-	dummy,			// 1
-	dummy,			// 2
-	dummy,			// 3
-	dummy,			// 4
-	dummy,			// 5
-	dummy,			// 6
-	dummy,			// 7
-	dummy,			// 8
-	dummy,			// 9
-	dummy,			// A
+	dummy,				// 0
+	dummy,				// 1
+	dummy,				// 2
+	dummy,				// 3
+	dummy,				// 4
+	dummy,				// 5
+	dummy,				// 6
+	dummy,				// 7
+	dummy,				// 8
+	dummy,				// 9
+	dummy,				// A
 	positionJump,		// B
-	dummy,			// C
+	dummy,				// C
 	patternBreak,		// D
 	E_Effects_TickZero,	// E
-	setSpeed,		// F
+	setSpeed,			// F
 	setGlobalVolume,	// G
-	dummy,			// H
-	dummy,			// I
-	dummy,			// J
-	dummy,			// K
+	dummy,				// H
+	dummy,				// I
+	dummy,				// J
+	dummy,				// K
 	setEnvelopePos,		// L
-	dummy,			// M
-	dummy,			// N
-	dummy,			// O
-	dummy,			// P
-	dummy,			// Q
-	dummy,			// R
-	dummy,			// S
-	dummy,			// T
-	dummy,			// U
-	dummy,			// V
-	dummy,			// W
-	dummy,			// X
-	dummy,			// Y
-	dummy 			// Z
+	dummy,				// M
+	dummy,				// N
+	dummy,				// O
+	dummy,				// P
+	dummy,				// Q
+	dummy,				// R
+	dummy,				// S
+	dummy,				// T
+	dummy,				// U
+	dummy,				// V
+	dummy,				// W
+	dummy,				// X
+	dummy,				// Y
+	dummy 				// Z
 };
 
 static void handleMoreEffects_TickZero(channel_t *ch) // called even if channel is muted!
@@ -1758,7 +1771,7 @@ static void arpeggio(channel_t *ch, uint8_t param)
 {
 	uint8_t note;
 
-	const uint8_t tick = arpeggioTab[song.tick & 255];
+	const uint8_t tick = arpeggioTab[song.tick & 31];
 	if (tick == 0)
 	{
 		ch->outPeriod = ch->realPeriod;
@@ -2261,12 +2274,12 @@ void tickReplayer(void) // periodically called from audio callback
 	}
 
 	// for song playback counter (hh:mm:ss)
-	if (song.BPM >= MIN_BPM && song.BPM <= MAX_BPM)
+	if (song.BPM >= MIN_BPM && song.BPM <= MAX_BPM) // just in case
 	{
-		song.playbackSecondsFrac += musicTimeTab52[song.BPM-MIN_BPM];
-		if (song.playbackSecondsFrac >= 1ULL << 52)
+		song.playbackSecondsFrac += songTickDuration35fp[song.BPM-MIN_BPM];
+		if (song.playbackSecondsFrac >= 1ULL << 35)
 		{
-			song.playbackSecondsFrac &= (1ULL << 52)-1;
+			song.playbackSecondsFrac &= (1ULL << 35)-1;
 			song.playbackSeconds++;
 		}
 	}
@@ -3132,6 +3145,7 @@ void stopVoices(void)
 	editor.curPlaySmp = 255;
 
 	stopAllScopes();
+	resetAudioDither();
 
 	// wait for scope thread to finish, making sure pointers aren't illegal
 	while (editor.scopeThreadBusy);
